# Some initial code which is the same for all the variants
import os
import csv
import numpy as np
import pandas as pd
import tensorflow as tf
from tensorboard.plugins import projector
from tqdm import tqdm
from PIL import Image
IMAGE_SIZE = 100




def get_img(img_path):
    img = tf.io.read_file(img_path)
    # convert the compressed string to a 3D uint8 tensor
    img = tf.image.decode_jpeg(img, channels=3)
    # resize the image to the desired size for your model
    img = tf.image.resize_with_pad(img, IMAGE_SIZE, IMAGE_SIZE)
    return img

def register_embedding(embedding_tensor_name, meta_data_fname, log_dir, sprite_path, with_images=True):
    config = projector.ProjectorConfig()
    embedding = config.embeddings.add()
    embedding.tensor_name = embedding_tensor_name
    embedding.metadata_path = meta_data_fname
    if (with_images):
        embedding.sprite.image_path = os.path.basename(sprite_path)
        embedding.sprite.single_image_dim.extend([IMAGE_SIZE, IMAGE_SIZE])
    projector.visualize_embeddings(log_dir, config)


def save_labels_tsv(labels, filepath, log_dir):
    with open(os.path.join(log_dir, filepath), 'w') as f:
        for label in labels:
            f.write('{}\n'.format(label))




def export_to_tensorboard_projector_inner(imglist, features, log_dir, sample_size, sample_method='random', with_images=True):
    try:
        if not os.path.exists(log_dir):
            os.mkdir(log_dir)
        
    except:
        print('Failed to create log_dir')
        return 1
    df = pd.DataFrame({'filenames':imglist})

    if (sample_method == 'random'):
        sample = df.sample(min(sample_size, len(df))).reset_index()
    else:
        print('sample method', sample_method, 'is not supported')
        return 

    META_DATA_FNAME = 'meta.tsv'  # Labels will be stored here
    EMBEDDINGS_TENSOR_NAME = 'embeddings'
    EMBEDDINGS_FPATH = os.path.join(log_dir, EMBEDDINGS_TENSOR_NAME + '.ckpt')
    STEP = 0

    # Generate embeddings
    images_pil = []
    images_embeddings = []
    labels = []
    for i,row in tqdm(sample.iterrows(), total=sample_size):

        img_path = row['filenames']
        img_tf = get_img(img_path)
        # Save both tf image for prediction and PIL image for sprite
        img_pil = Image.open(img_path).resize((IMAGE_SIZE, IMAGE_SIZE))
        img_embedding = features[row['index'], :]
        images_embeddings.append(img_embedding)
        images_pil.append(img_pil)
        # Assuming your output data is directly the label
        label = img_path.split('/')[2]
        labels.append(label)

    NUM_IMAGES_WIDTH = int(np.ceil(np.sqrt(sample_size)))

    # Create a sprite image
    spriteimage = Image.new(
        mode='RGB',
        size=(IMAGE_SIZE*NUM_IMAGES_WIDTH, IMAGE_SIZE*NUM_IMAGES_WIDTH),
        color=(0,0,0) # fully transparent
    )
    for count, image in enumerate(images_pil):
        h_loc = count // NUM_IMAGES_WIDTH
        w_loc = count % NUM_IMAGES_WIDTH
        spriteimage.paste(image, (w_loc*IMAGE_SIZE, h_loc*IMAGE_SIZE))

    SPRITE_PATH= f'{log_dir}/sprite.png'
    spriteimage.convert('RGB').save(SPRITE_PATH)


    register_embedding(EMBEDDINGS_TENSOR_NAME, META_DATA_FNAME, log_dir, SPRITE_PATH, with_images)
    save_labels_tsv(labels, META_DATA_FNAME, log_dir)

    # Size of files created on disk: 80.5kB
    ids = sample['index'].values
    assert len(ids)
    assert features.shape[1] == 576

    tensor_embeddings = tf.Variable(features[ids,:], name=EMBEDDINGS_TENSOR_NAME)
    saver = tf.compat.v1.train.Saver([tensor_embeddings])  # Must pass list or dict
    saver.save(sess=None, global_step=STEP, save_path=EMBEDDINGS_FPATH)

    print('Finish exporting to tensorboard projector, now run')
    if 'JPY_PARENT_PID' in os.environ:
        print('%load_ext tensorboard')
        print(f'%tensorboard --logdir={log_dir}')
    else:
        print(f'tensorboard --logdir={log_dir}')
